@@ -15,7 +15,7 @@ urllib3.disable_warnings()
 class API:
     url: str
     server_info = {}
-    version:str = "0.0.1"
+    version:str = "0.0.2"
 
     def __post_init__(self):
         try:
@@ -45,11 +45,11 @@ class API:
                        ,"bondPricing": deal.read_pricing(pricing) if (pricing is not None) else None}
                    , ensure_ascii=False)
 
-        if any(isinstance(i, list) for i in assumptions):
+        if isinstance(assumptions, dict):
         #if isinstance(assumptions,list):
             return json.dumps({"_deal": deal.json
                        ,"_assump": mkTag(("Multiple"
-                                          ,[ deal.read_assump(a) for a in assumptions ]))
+                                          ,{ scenarioName:deal.read_assump(a) for (scenarioName,a) in assumptions.items()}))
                        ,"_bondPricing": deal.read_pricing(pricing)}
                    , ensure_ascii=False)
 
@@ -68,6 +68,10 @@ class API:
         valid_bnd = set(_d['bonds'].keys())
         valid_fee = set(_d['fees'].keys())
         _w = _d['waterfall']
+
+        if _w is None:
+            raise RuntimeError("Waterfall is None")
+
         for wn,wa in _w.items():
             for idx,action in enumerate(wa):
                 #print(action)
@@ -89,6 +93,10 @@ class API:
                         if (action['contents'][0] not in valid_acc) \
                             or (not set(action['contents'][1]).issubset(valid_bnd)):
                             error.append(f"{wn},{idx}")  
+                    case 'PayPrinBy':
+                        if (action['contents'][1] not in valid_acc) \
+                            or (not set(action['contents'][2]).issubset(valid_bnd)):
+                            error.append(f"{wn},{idx}")  
                     case 'PayResidual':
                         if (action['contents'][1] not in valid_acc) \
                             or (action['contents'][2] not in valid_bnd):
@@ -98,8 +106,8 @@ class API:
                             or (action['contents'][1] not in valid_acc):
                             error.append(f"{wn},{idx}")
                     case 'TransferBy':
-                        if (action['contents'][0] not in valid_acc) \
-                            or (action['contents'][1] not in valid_acc):
+                        if (action['contents'][1] not in valid_acc) \
+                            or (action['contents'][2] not in valid_acc):
                             error.append(f"{wn},{idx}")
                     case 'PayTillYield':
                         if (action['contents'][0] not in valid_acc) \
@@ -109,11 +117,11 @@ class API:
                         if (action['contents'][1] not in valid_acc) \
                             or (action['contents'][2] not in valid_fee):
                             error.append(f"{wn},{idx}")        
-        _d = _r[_deal_key]['dates']
-        if _d['ClosingDate'] >= _d['FirstPayDate']:
-            error.append(f"dates,first pay date/next pay date should be after closing date")
-        if _d['CutoffDate'] >= _d['FirstPayDate']:
-            error.append(f"dates,first pay date/next pay date should be after cutoff date")
+        # _d = _r[_deal_key]['dates']
+        # if _d['ClosingDate'] >= _d['FirstPayDate']:
+        #     error.append(f"dates,first pay date/next pay date should be after closing date")
+        # if _d['CutoffDate'] >= _d['FirstPayDate']:
+        #     error.append(f"dates,first pay date/next pay date should be after cutoff date")
 
         if warning:
             logging.warning(f"Warning in modelling:{warning}")
@@ -139,7 +147,7 @@ class API:
             assumptions = pickle.load(assumptions)
 
         if assumptions:
-            multi_run_flag = any(isinstance(i, list) for i in assumptions)
+            multi_run_flag = isinstance(assumptions, dict)
         else:
             multi_run_flag = False 
             
@@ -188,7 +196,7 @@ class API:
         t_reading_s = datetime.datetime.now()
         if read:
             if multi_run_flag:
-                __r = [ deal.read(_r,position=position) for _r in result ]
+                __r = { n:deal.read(_r,position=position) for (n,_r) in result.items()}
             else:
                 __r = deal.read(result,position=position)
             t_reading_e = datetime.datetime.now()
@@ -211,3 +219,38 @@ def init_jupyter():
     pd.set_option('display.width', None)
     pd.set_option('display.max_colwidth', None)
     pd.options.display.float_format = '{:,}'.format
+
+def comp_engines(engine1,engine2, d, a=None):
+    
+    r1 = engine1.run(d,assumptions=a,read=True)
+    r2 = engine2.run(d,assumptions=a,read=True)
+
+    #pool check
+    if not r1['pool']['flow'] == r2['pool']['flow']:
+        return r1['pool']['flow'].compare(r2['pool']['flow']) 
+
+    #expense check  
+    if not r1['fees'] == r2['fees']:
+        for fn,f in r1['fees'].items():
+            if f.equals(r2['fees'][fn]):
+                continue
+            else:
+                return f.compare(r2['fees'][fn])
+
+    #bond check
+    if not r1['bonds'] == r2['bonds']:
+        for fn,f in r1['bonds'].items():
+            if f.equals(r2['bonds'][fn]):
+                continue
+            else:
+                return f.compare(r2['bonds'][fn])
+
+
+    #account check
+    if not r1['accounts'] == r2['accounts']:
+        for fn,f in r1['accounts'].items():
+            if f.equals(r2['accounts'][fn]):
+                continue
+            else:
+                return f.compare(r2['accounts'][fn])
+
