@@ -3,173 +3,17 @@ import requests, shutil
 from dataclasses import dataclass,field
 import functools, pickle, collections
 import pandas as pd
+import numpy as np
 from urllib.request import unquote
 from enum import Enum
+from functools import reduce 
+import matplotlib.pyplot as plt
 
 from absbox import *
-from absbox.local.util import mkTag
+from absbox.local.util import mkTag,DC,mkTs,query,consolStmtByDate,aggStmtByDate
 
+from absbox.local.component import *
 
-class 频率(Enum):
-    每月 = 12
-    每季度 = 4
-    每半年 = 2
-    每年 = 1
-
-
-freqMap = {"每月": "Monthly"
-    , "每季度": "Quarterly"
-    , "每半年": "SemiAnnually"
-    , "每年": "Annually"}
-
-baseMap = {"资产池余额": "CurrentPoolBalance"
-           , "资产池期末余额": "CurrentPoolBalance"
-           , "资产池期初余额": "CurrentPoolBegBalance"
-           , "资产池初始余额": "OriginalPoolBalance"
-           , "资产池当期利息":"PoolCollectionInt"
-           , "债券余额":"CurrentBondBalance"
-           , "债券初始余额":"OriginalBondBalance"
-           , "当期已付债券利息":"LastBondIntPaid"
-           , "当期已付费用" :"LastFeePaid"
-           , "当期未付债券利息" :"CurrentDueBondInt"
-           , "当期未付费用": "CurrentDueFee"
-           }
-
-def mkBondType(x):
-    match x:
-        case {"固定摊还": schedule}:
-            return mkTag(("PAC", mkTag(("AmountCurve", schedule))))
-        case {"过手摊还": None}:
-            return mkTag(("Sequential"))
-        case {"锁定摊还": _after}:
-            return mkTag(("Lockout", _after))
-        case {"权益": _}:
-            return mkTag(("Equity"))
-
-
-def mkAccType(x):
-    match x:
-        case {"固定储备金额": amt}:
-            return mkTag(("FixReserve", amt))
-        case {"目标储备金额": [base, rate]}:
-            match base:
-                case ["合计",*q]:
-                    return mkTag(("PctReserve"
-                                 , [mkTag(("Sum"
-                                           ,[mkTag((baseMap[_b], _ts)) for (_b, _ts) in q]))
-                                   , rate ]))
-                case _ :
-                    return mkTag(("PctReserve", [mkTag((baseMap[base])), rate]))
-        case {"较高": [a, b]}:
-            return mkTag(("Max", [mkAccType(a), mkAccType(b)]))
-        case {"较低": [a, b]}:
-            return mkTag(("Min", [mkAccType(a), mkAccType(b)]))
-
-
-def mkFeeType(x):
-    match x:
-        case {"年化费率": [base, rate]}:
-            return mkTag(("AnnualRateFee"
-                        ,[ mkTag((baseMap[base],'1970-01-01')) 
-                           , rate]))
-        case {"百分比费率": [base, rate]}:
-            return mkTag(("PctFee"
-                        ,[mkTag(baseMap[base])
-                          , rate]))
-        case {"固定费用": amt}:
-            return mkTag(("FixFee", amt))
-        case {"周期费用": [p, amt]}:
-            return mkTag(("RecurFee", [freqMap[p], amt]))
-
-
-def mkRateReset(x):
-    match x:
-        case {"重置期间": interval, "起始": sdate}:
-            return mkTag(("ByInterval", [freqMap[interval], sdate]))
-        case {"重置期间": interval}:
-            return mkTag(("ByInterval", [freqMap[interval], None]))
-        case {"重置月份": monthOfYear}:
-            return mkTag(("MonthOfYear", monthOfYear))
-
-
-def mkBondRate(x):
-    indexMapping = {"LPR5Y": "LPR5Y", "LIBOR1M": "LIBOR1M"}
-    match x:
-        case {"浮动": [_index, Spread, resetInterval]}:
-            return {"tag": "Floater"
-                , "contents": [indexMapping[_index]
-                    , Spread
-                    , mkRateReset(resetInterval)
-                    , None
-                    , None]}
-        case {"固定": _rate}:
-            return mkTag(("Fix",_rate))
-        case {"期间收益": _yield}:
-            return mkTag(("InterestByYield",_yield))
-
-
-def mkFeeCapType(x):
-    match x:
-        case {"应计费用百分比": pct}:
-            return mkTag(("DuePct",pct))
-        case {"应计费用上限": amt}:
-            return mkTag(("DueCapAmt",amt))
-
-def mkAccountCapType(x):
-    match x:
-        case {"余额百分比": pct}:
-            return mkTag(("DuePct",pct))
-        case {"金额上限": amt}:
-            return mkTag(("DueCapAmt",amt))
-
-    
-def mkFormula(x):
-    match x:
-        case "A+B+C-D":
-            return mkTag(("ABCD"))
-
-def mkLiqMethod(x):
-    match x:
-        case ["正常|违约",a,b]:
-            return mkTag(("BalanceFactor",[a,b]))
-        case ["正常|拖欠|违约",a,b,c]:
-            return mkTag(("BalanceFactor2",[a,b,c]))
-        case ["贴现|违约",a,b]:
-            return mkTag(("PV",[a,b]))
-
-def mkAction(x):
-    match x:
-        case ["账户转移", source, target]:
-            return mkTag(("Transfer",[source, target, ""]))
-        case ["按公式账户转移", source, target, formula]:
-            return mkTag(("TransferBy",[source, target, mkFormula(formula)]))
-        case ["支付费用", source, target]:
-            return mkTag(("PayFee",[source, target]))
-        case ["支付费用收益", source, target, _limit]:
-            limit = mkAccountCapType(_limit)
-            return mkTag(("PayFeeResidual",[limit, source, target]))
-        case ["支付费用收益", source, target]:
-            return mkTag(("PayFeeResidual",[None, source, target]))
-        case ["支付费用限额", source, target, _limit]:
-            limit = mkFeeCapType(_limit)
-            return mkTag(("PayFeeBy",[limit, source, target]))
-        case ["支付利息", source, target]:
-            return mkTag(("PayInt",[source, target]))
-        case ["支付本金", source, target]:
-            return mkTag(("PayPrin",[source, target]))
-        case ["支付剩余本金", source, target]:
-            return mkTag(("PayPrinResidual",[source, target]))
-        case ["支付期间收益", source, target]:
-            return mkTag(("PayTillYield",[source, target]))
-        case ["支付收益", source, target, limit]:
-            return mkTag(("PayResidual",[limit, source, target]))
-        case ["支付收益", source, target]:
-            return mkTag(("PayResidual",[None, source, target]))
-        case ["储备账户转移", source, target, satisfy]:
-            _map = {"源储备":"Source","目标储备":"Target"}
-            return mkTag(("TransferReserve",[_map[satisfy], source, target, None]))
-        case ["出售资产", liq, target]:
-            return mkTag(("LiquidatePool",[mkLiqMethod(liq), target]))
 
 #data DealStats =
 #              | CurrentPoolDefaultedBalance
@@ -181,81 +25,6 @@ def mkAction(x):
 #              | LastFeePaid [String]
 
 
-def mkDs(x):
-    "Making Deal Stats"
-    match x:
-        case ("债券余额",):
-            return mkTag("CurrentBondBalance")
-        case ("资产池余额",):
-            return mkTag("CurrentPoolBalance")
-        case ("初始债券余额",):
-            return mkTag("OriginalBondBalance")
-        case ("初始资产池余额",):
-            return mkTag("OriginalPoolBalance")
-        case ("债券系数",):
-            return mkTag("BondFactor")
-        case ("资产池系数",):
-            return mkTag("PoolFactor")
-        case ("所有账户余额",):
-            return mkTag("AllAccBalance")
-        case ("系数",ds,f):
-            return mkTag(("Factor",[mkDs(ds),f]))
-        case ("债券余额",*bnds):
-            return mkTag(("CurrentBondBalanceOf",bnds))
-        case ("Min",ds1,ds2):
-            return mkTag(("Min",[mkDs(ds1),mkDs(ds2)]))
-        case ("Max",ds1,ds2):
-            return mkTag(("Max",[mkDs(ds1),mkDs(ds2)]))
-        case ("合计",*ds):
-            return mkTag(("Sum",[mkDs(_ds) for _ds in ds]))
-            
-
-
-def mkPre(p):
-    dealStatusMap = {"摊还":"Current"
-                     ,"加速清偿":"Accelerated"
-                     ,"循环":"Revolving"}
-    match p:
-        case [ds,">",amt]:
-            return mkTag(("IfGT",[mkDs(ds),amt]))
-        case [ds,"<",amt]:
-            return mkTag(("IfLT",[mkDs(ds),amt]))
-        case [ds,">=",amt]:
-            return mkTag(("IfGET",[mkDs(ds),amt]))
-        case [ds,"<=",amt]:
-            return mkTag(("IfLET",[mkDs(ds),amt]))
-        case [ds,"=",0]:
-            return mkTag(("IfZero",mkDs(ds)))
-        case ["状态",_ds]:
-            return mkTag(("IfDealStatus",dealStatusMap[_ds]))
-        case ["同时满足",_p1,_p2]:
-            return mkTag(("And",mkPre(_p1),mkPre(_p2)))
-        case ["任一满足",_p1,_p2]:
-            return mkTag(("Or",mkPre(_p1),mkPre(_p2)))
-
-def isPre(x):
-    return mkPre(x) is not None
-
-def mkWaterfall(x):
-    match x:
-        case (pre,_action): 
-            action = mkAction(_action)
-            return [mkPre(pre),action]
-        case _:
-            return [None,mkAction(x)]
-
-def mkWaterfall2(x):
-    match x:
-        case (pre, *_action) if isPre(pre) and len(x)>2: # pre with multiple actions
-            _pre = mkPre(pre)
-            return [[ _pre, mkAction(a) ] for a in _action ]
-        case (pre, _action) if isPre(pre) and len(x)==2: # pre with multiple actions
-            _pre = mkPre(pre)
-            return [[ _pre, mkAction(_action) ]]
-        case _:
-            return [[ None,mkAction(x) ]]
-
-
 def mkAssetRate(x):
     match x:
         case ["固定",r]:
@@ -313,22 +82,35 @@ def mkCollection(xs):
     return [[sourceMapping[x], acc] for (x, acc) in xs]
 
 
-def mkDate(x):
+def mkLiqProviderType(x):
     match x:
-        case (a,b,c):
-            return [a,b,c]
-        case {"封包日":a, "起息日": b,"首次兑付日":c}:
-            return [a,b,c]
-        case {"收款期末日":a,"计息期末日": b,"下次兑付日":c}:
-            return [a,b,c]
+        case {"总额度": amt}:
+            return mkTag(("FixSupport"))
+        case {"日期":dp, "限额":amt}:
+            return mkTag(("ReplenishSupport", [mkDatePattern(dp),amt]))
+        case {}:
+            return mkTag(("UnLimit"))
+
+def mkLiqProvider(n, x):
+    match x:
+        case {"类型":"无限制","起始日":_sd, **p}: 
+            return {"liqName": n, "liqType": mkLiqProviderType({})
+                   , "liqBalance": None
+                   , "liqCredit": p.get("已提供", 0)
+                   , "liqStart": _sd}
+        case {"类型": _sp, "额度": _ab, "起始日":_sd, **p}: 
+            return {"liqName": n, "liqType": mkLiqProviderType(_sp)
+                   , "liqBalance": _ab
+                   , "liqCredit": p.get("已提供", 0)
+                   , "liqStart": _sd}
         case _:
-            raise RuntimeError(f"对于产品发行建模格式为：{'封包日':a, '起息日': b,'首次兑付日':c} ;对于存量产品格式为 {'收款期末日':a,'计息期末日': b,'下次兑付日':c} ")
+            raise RuntimeError(f"无法匹配流动性支持类型：{n,x}")
 
 
 def mkComponent(x):
     match x:
         case {"贴现日": pricingDay, "贴现曲线": xs}:
-            return [pricingDay, {"tag": "FloatCurve", "contents": xs}]
+            return [pricingDay, {"tag": "PricingCurve", "contents": xs}]
         case _:
             None
 
@@ -365,8 +147,12 @@ def mkAssumption(x):
             return mkTag(("PrepaymentCPRCurve", cpr))
         case {"CPR": cpr} :
             return mkTag(("PrepaymentCPR", cpr))
+        case {"CPR调整": [*cprAdj,eDate]} :
+            return mkTag(("PrepaymentFactors" , mkTs("FactorCurveClosed",[cprAdj,eDate])))
         case {"CDR": cdr}:
             return mkTag(("DefaultCDR", cdr))
+        case {"CDR调整": [*cdrAdj,eDate]} :
+            return mkTag(("DefaultFactors" , mkTs("FactorCurveClosed",[cdrAdj,eDate])))
         case {"回收": (rr, rlag)}:
             return mkTag(("Recovery", (rr, rlag)))
         case {"利率": [idx, rate]} if isinstance(rate, float):
@@ -385,76 +171,141 @@ def mkAccTxn(xs):
     else:
         return [ mkTag(("AccTxn",x)) for x in xs]
 
+
+def mkComponent(x):
+    match x:
+        case {"贴现日": pricingDay, "贴现曲线": xs}:
+            return [pricingDay, {"tag": "PricingCurve", "contents": xs}]
+        case _:
+            None
+
+
+def mkLiq(x):
+    match x:
+        case {"正常余额折价": cf, "违约余额折价": df}:
+            return mkTag(("BalanceFactor", [cf, df]))
+        case {"贴现计价": df, "违约余额回收率": r}:
+            return mkTag(("PV", [df, r]))
+
+
+def mkCallOptions(x):
+    match x:
+        case {"资产池余额": bal}:
+            return mkTag(("PoolBalance", bal))
+        case {"债券余额": bal}:
+            return mkTag(("BondBalance", bal))
+        case {"资产池余额剩余比率": factor}:
+            return mkTag(("PoolFactor", factor))
+        case {"债券余额剩余比率": factor}:
+            return mkTag(("BondFactor", factor))
+        case {"指定日之后": d}:
+            return mkTag(("AfterDate", d))
+        case {"任意满足": xs}:
+            return mkTag(("Or", xs))
+        case {"全部满足": xs}:
+            return mkTag(("And", xs))
+
+
+def mkAssumption(x):
+    match x:
+        case {"CPR": cpr} if isinstance(cpr, list):
+            return mkTag(("PrepaymentCPRCurve", cpr))
+        case {"CPR": cpr} :
+            return mkTag(("PrepaymentCPR", cpr))
+        case {"CPR调整": [*cprAdj,eDate]} :
+            return mkTag(("PrepaymentFactors" , mkTs("FactorCurveClosed",[cprAdj,eDate])))
+        case {"CDR": cdr}:
+            return mkTag(("DefaultCDR", cdr))
+        case {"CDR调整": [*cdrAdj,eDate]} :
+            return mkTag(("DefaultFactors" , mkTs("FactorCurveClosed",[cdrAdj,eDate])))
+        case {"回收": (rr, rlag)}:
+            return mkTag(("Recovery", (rr, rlag)))
+        case {"利率": [idx, rate]} if isinstance(rate, float):
+            return mkTag(("InterestRateConstant", [idx, rate]))
+        case {"利率": [idx, *rateCurve]}:
+            return mkTag(("InterestRateCurve", [idx, *rateCurve]))
+        case {"清仓": opts}:
+            return mkTag(("CallWhen",[mkCallOptions(co) for co in opts]))
+        case {"停止": d}:
+            return mkTag(("StopRunBy",d))
+
 # \"overrides\":[[{\"tag\":\"RunWaterfall\",\"contents\":[\"2022-01-01\",\"base\"]},{\"tag\":\"PoolCollection\",\"contents\":[\"0202-11-01\",\"collection\"]}]]}
 def mkOverrides(m):
-    r = []
-    mapping = {"分配日":"RunWaterfall","回款日":"PoolCollection"}
-    for k,v in m.items():
-        match (k,v):
-            case("日期",ds):
-                r.append([mkTag((mapping[_x[0]],[_x[1],""])) for _x in ds ])
-    return r
+    return None
+#data WhenTrigger = EndCollection
+#                 | EndCollectionWF
+#                 | BeginDistributionWF
+#                 | EndDistributionWF
+
+# [
+#  [[\"BeginDistributionWF\",{\"tag\":\"AfterDate\",\"contents\":\"2022-03-01\"}]
+#    ,{\"tag\":\"DealStatusTo\",\"contents\":{\"tag\":\"Revolving\"}}]
+#   ]"
+
+def mkCustom(x):
+    match x:
+        case {"常量":n}:
+            return mkTag(("CustomConstant",n))
+        case {"余额曲线":ts}:
+            return mkTag(("CustomCurve",mkTs("BalanceCurve",ts)))
+        case {"公式":ds}:
+            return mkTag(("CustomDS",mkDs(ds)))
 
 def mk(x):
     match x:
         case ["资产", assets]:
             return {"assets": [mkAsset(a) for a in assets]}
         case ["账户", accName, attrs]:
-            match attrs:
-                case {"余额": bal, "类型": accType}:
-                    return {accName: {"accBalance": bal, "accName": accName
-                                      , "accType": mkAccType(accType)
-                                      , "accInterest": None
-                                      , "accStmt": mkAccTxn(attrs.get("记录",None))}}
-                case {"余额": bal}:
-                    return { accName: {"accBalance": bal, "accName": accName
-                                      , "accType": None, "accInterest": None
-                                      , "accStmt": mkAccTxn(attrs.get("记录",None))}}
-        case ["费用", feeName, {"类型": feeType}]:
-            return {feeName: {"feeName": feeName, "feeType": mkFeeType(feeType), "feeStart": None, "feeDue": 0,
+            return {accName: mkAcc(accName, attrs)}
+            #match attrs:
+            #    case {"余额": bal, "类型": accType}:
+            #        return {accName: {"accBalance": bal, "accName": accName
+            #                          , "accType": mkAccType(accType)
+            #                          , "accInterest": mkAccInt(attrs.get("计息",None))
+            #                          , "accStmt": mkAccTxn(attrs.get("记录",None))}}
+            #    case {"余额": bal}:
+            #        return { accName: {"accBalance": bal, "accName": accName
+            #                          , "accType": None
+            #                          , "accInterest": mkAccInt(attrs.get("计息",None))
+            #                          , "accStmt": mkAccTxn(attrs.get("记录",None))}}
+        case ["费用", feeName, {"类型": feeType ,**fi}]:
+            return {feeName: {"feeName": feeName, "feeType": mkFeeType(feeType), "feeStart":fi.get("起算日",None)
+                             ,"feeDueDate":fi.get("计算日",None) , "feeDue": 0,
                               "feeArrears": 0, "feeLastPaidDay": None}}
-        case ["债券", bndName, {"当前余额": bndBalance
-            , "当前利率": bndRate
-            , "初始余额": originBalance
-            , "初始利率": originRate
-            , "起息日": originDate
-            , "利率": bndInterestInfo
-            , "债券类型": bndType
-                              }]:
-            return {bndName:
-                        {"bndName": bndName
-                            , "bndBalance": bndBalance
-                            , "bndRate": bndRate
-                            , "bndOriginInfo":
-                             {"originBalance": originBalance
-                                 , "originDate": originDate
-                                 , "originRate": originRate}
-                            , "bndInterestInfo": mkBondRate(bndInterestInfo)
-                            , "bndType": mkBondType(bndType)
-                            , "bndDuePrin": 0
-                            , "bndDueInt": 0
-                         }}
-        case ["分配规则", instruction]:
-            return mkWaterfall(instruction)
+        case ["债券", bndName, bnd]:
+            return mkBnd(bndName, bnd)
         case ["归集规则", collection]:
             return mkCollection(collection)
         case ["清仓回购", calls]:
             return mkCall(calls)
 
+def readStatus(s):
+    if "," in s:
+        return s.split(",")[1]
+    else:
+        return mkTag("Amortizing")
+
+class 时间点(Enum):
+    回收后 = "BeginCollectionWF"
+    回收动作后 = "EndCollectionWF"
+    分配前 = "BeginDistributionWF"
+    分配后 = "EndDistributionWF"
 
 @dataclass
 class 信贷ABS:
     名称: str
-    日期: tuple  # 起息日: datetime 封包日: datetime 首次兑付日 : datetime
-    兑付频率: 频率
+    日期: dict
     资产池: dict
     账户: tuple
     债券: tuple
     费用: tuple
     分配规则: dict
     归集规则: tuple
-    清仓回购: tuple 
-    自定义: dict = field(default_factory=dict)
+    清仓回购: tuple = None
+    流动性支持:dict = None
+    自定义: dict = None
+    触发事件: dict = None
+    状态:str = "摊销"
 
 
     @classmethod
@@ -485,43 +336,59 @@ class 信贷ABS:
 
     @property
     def json(self):
-        cutoff, closing, first_pay = mkDate(self.日期)
+        #cutoff, closing, first_pay = mkDate(self.日期)
+        stated = False # self.日期.get("法定到期日",None) if len(self.日期)==4  # if isinstance(self.日期,dict) else self.日期[3]
         dists,collects,cleans = [ self.分配规则.get(wn,[]) for wn in ['未违约','回款后','清仓回购'] ]
         distsAs,collectsAs,cleansAs = [ [ mkWaterfall2(_action) for _action in _actions] for _actions in [dists,collects,cleans] ]
         distsflt,collectsflt,cleanflt = [ itertools.chain.from_iterable(x) for x in [distsAs,collectsAs,cleansAs] ]
+        parsedDates = mkDate(self.日期)
+        status = mkStatus(self.状态)
         """
         get the json formatted string
         """
         _r = {
-            "dates": {
-                "ClosingDate": closing,
-                "CutoffDate": cutoff,
-                "FirstPayDate": first_pay},
+            "dates": parsedDates,
             "name": self.名称,
-            "pool": {"assets": [mkAsset(x) for x in self.资产池.get('清单',[])]
-                , "asOfDate": cutoff
+            "status": status,
+            "pool":{"assets": [mkAsset(x) for x in self.资产池.get('清单',[])]
+                , "asOfDate": self.日期['封包日']
                 , "issuanceStat": readIssuance(self.资产池)
                 , "futureCf":mkCf(self.资产池.get('归集表', []))
                 },
             "bonds": functools.reduce(lambda result, current: result | current
                                       , [mk(['债券', bn, bo]) for (bn, bo) in self.债券]),
-            "waterfall": {"DistributionDay": list(distsflt)
-                        , "EndOfPoolCollection": list(collectsflt)
-                        , "CleanUp": list(cleanflt)},
+            #"waterfall": {f"DistributionDay {status['tag']}":list(distsflt)
+            #            , "EndOfPoolCollection": list(collectsflt)
+            #            , "CleanUp": list(cleanflt)},
+            "waterfall": mkWaterfall({},self.分配规则.copy()),
             "fees": functools.reduce(lambda result, current: result | current
                                      , [mk(["费用", feeName, feeO]) for (feeName, feeO) in self.费用]) if self.费用 else {},
             "accounts": functools.reduce(lambda result, current: result | current
                                          , [mk(["账户", accName, accO]) for (accName, accO) in self.账户]),
-            "collects": mkCollection(self.归集规则),
-            "collectPeriod": freqMap[self.兑付频率],
-            "payPeriod": freqMap[self.兑付频率],
+            "collects": mkCollection(self.归集规则)
         }
+        
         for fn, fo in _r['fees'].items():
-            fo['feeStart'] = _r['dates']['ClosingDate']
+            if fo['feeStart'] is None :
+                fo['feeStart'] = self.日期["起息日"]
 
-        if hasattr(self,"自定义"):
-            _r["overrides"] = mkOverrides(self.自定义)
-            
+        if hasattr(self, "自定义") and self.自定义 is not None:
+            _r["custom"] = {}
+            for n,ci in self.自定义.items():
+                _r["custom"][n] = mkCustom(ci)
+        
+        if hasattr(self, "触发事件") and self.触发事件 is  not None:
+            _trigger  = self.触发事件
+            _trr = {mkWhenTrigger(_loc):
+                       [[mkTrigger(_trig),mkTriggerEffect(_effect)] for (_trig,_effect) in _vs ] 
+                       for _loc,_vs in _trigger.items()}
+            _r["triggers"] = _trr
+        
+        if hasattr(self, "流动性支持") and self.流动性支持 is not None:
+            _providers = {}
+            for (_k, _p) in self.流动性支持.items():
+                _providers[_k] = mkLiqProvider(_k, ( _p | {"起始日": self.日期["起息日"]}))
+            _r["liqProvider"] = _providers
         return _r  # ,ensure_ascii=False)
 
     def _get_bond(self, bn):
@@ -543,10 +410,13 @@ class 信贷ABS:
     def read(self, resp, position=None):
         read_paths = {'bonds': ('bndStmt', ["日期", "余额", "利息", "本金", "执行利率", "本息合计", "备注"], "债券")
                     , 'fees': ('feeStmt', ["日期", "余额", "支付", "剩余支付", "备注"], "费用")
-                    , 'accounts': ('accStmt', ["日期", "余额", "变动额", "备注"], "账户")}
+                    , 'accounts': ('accStmt', ["日期", "余额", "变动额", "备注"], "账户")
+                    , 'liqProvider': ('liqStmt', ["日期", "限额", "变动额", "已提供","备注"], "流动性支持")
+                    }
         output = {}
         for comp_name, comp_v in read_paths.items():
-            #output[comp_name] = collections.OrderedDict()
+            if (not comp_name in resp[0]) or (resp[0][comp_name] is None):
+                continue
             output[comp_name] = {}
             for k, x in resp[0][comp_name].items():
                 ir = None
@@ -558,6 +428,11 @@ class 信贷ABS:
         output['fees'] = {f: v.groupby('日期').agg({"余额": "min", "支付": "sum", "剩余支付": "min"})
                           for f, v in output['fees'].items()}
 
+        # aggregate liquidation provider 
+        #if 'liqProvider' in output:
+        #    output['liqProvider'] = {l:v.   
+        #                             for l,v in output['liqProvider'].items() }
+
         # aggregate accounts
         agg_acc = {}
         for k,v in output['accounts'].items():
@@ -583,7 +458,7 @@ class 信贷ABS:
 
         output['pricing'] = pd.DataFrame.from_dict(resp[3]
                                                    , orient='index'
-                                                   , columns=["估值", "票面估值", "WAL", "久期", "应计利息"]) if resp[3] else None
+                                                   , columns=["估值", "票面估值", "WAL", "久期", "应计利息"]).sort_index() if resp[3] else None
         if position:
             output['position'] = {}
             for k,v in position.items():
@@ -625,6 +500,24 @@ def show(r, x="full"):
         case "cash":
             return None # ""
 
+def flow_by_scenario(rs, flowpath,annotation=True,aggFunc=None,rnd=2):
+    "pull flows from multiple scenario"
+    scenario_names = rs.keys()
+    dflow = None
+    aggFM = {"max":pd.Series.max,"sum":pd.Series.sum,"min":pd.Series.min}
+    
+    if aggFunc is None:
+        dflows = [query(rs,[s]+flowpath) for s in scenario_names]
+    else:
+        dflows = [query(rs,[s]+flowpath).groupby("日期").aggregate(aggFM.get(aggFunc,aggFunc)) for s in scenario_names]
+        
+    if annotation:
+        dflows = [f.rename(f"{s}({flowpath[-1]})") for (s,f) in zip(scenario_names,dflows)]
+    try: 
+        return pd.concat(dflows,axis=1).round(rnd)
+    except ValueError as e:
+        return f"需要传入 aggFunc 函数对重复数据进行 Min/Max/Sum 处理"
+
 
 import matplotlib.pyplot as plt
 from matplotlib import font_manager
@@ -683,3 +576,31 @@ def plot_bonds(r, bnds:list, flow='本息合计'):
     current_values = plt.gca().get_yticks()
     plt.gca().set_yticklabels(['{:.0f}(w)'.format(x/10000) for x in current_values])
     return plt
+
+def plot_by_scenario(rs, flowtype, flowpath):
+    "Plot with multiple scenario"
+    plt.figure(figsize=(12,8))
+    scenario_names = rs.keys()
+    dflows = [query(rs,[s]+flowpath) for s in scenario_names]
+    _alpha =  0.8
+
+    x_labels = reduce(lambda acc,x:acc.union(x) ,[ _.index for _ in dflows ]).unique()
+    x = np.arange(len(x_labels))
+    width = 1 
+    step_length = width / (len(scenario_names)+1)
+
+    for (idx,(scen,dflow)) in enumerate(zip(scenario_names,dflows)):
+        if flowtype=="balance":
+            cb = consolStmtByDate(dflow)
+            plt.step(cb.index, cb, alpha=_alpha, linewidth=5, label=f"{scen}")
+        elif flowtype=="amount":
+            cb = aggStmtByDate(dflow)
+            _bar = plt.bar(x+idx*step_length,cb,width=step_length,label=scen)
+        else:
+            plt.plot(dflow.index,dflow, alpha=_alpha, linewidth=5, label=f"{scen}")
+
+    plt.legend(scenario_names,loc='upper right', prop=font_p)
+    plt.grid(True)
+    plt.axis('tight')
+    plt.xticks(ticks=x,labels=x_labels,rotation=30)
+
